@@ -49,11 +49,14 @@ from archive_handlers import ZipfileHandler, TarfileHandler, create_handler, \
 from django_config import DjangoConfig, django_config_from_json
 from generate_settings import generate_settings_file, simulated_install
 from utils import app_module_name_to_dir, write_json, find_files, \
-                  get_deployed_settings_module
+                  get_deployed_settings_module, import_module
 from validation_results import *
 import command
 import parse_requirements
 from find_module import find_python_module
+from engage_django_components import validate_django_components
+from package_data import PREINSTALLED_PACKAGES, get_apps_for_packages, \
+     extend_app_list, PACKAGES_FOR_KNOWN_APPS
 
 verbose = False
 
@@ -88,6 +91,13 @@ def run_safe_validations(handler):
     return (common_dir, django_config)
 
 
+def run_safe_validations_on_archive(archive_file):
+    """Wrapper on run_safe_validations that opens the archive,
+    does the validations, and then closes it.
+    """
+    with create_handler(archive_file) as handler:
+        return run_safe_validations(handler)
+
 def check_if_setting_overridden(setting_name, module, results):
     if not hasattr(module, setting_name):
         raise ValidationError("Expected setting '%s' not defined. Is the setting file set up correctly?" % setting_name)
@@ -145,30 +155,6 @@ def check_directory_tuple_setting(setting_name, setting_value,
                                     None, app_dir_path, results)
 
 
-PREINSTALLED_APPS = [
-    "django.contrib.admin",
-    "django.contrib.admindocs",
-    "django.contrib.auth",
-    "django.contrib.comments",
-    "django.contrib.contenttypes",
-    "django.contrib.csrf",
-    "django.contrib.databrowse",
-    "django.contrib.flatpages",
-    "django.contrib.formtools",
-    "django.contrib.gis",
-    "django.contrib.humanize",
-    "django.contrib.localflavor",
-    "django.contrib.markup",
-    "django.contrib.messages",
-    "django.contrib.redirects",
-    "django.contrib.sessions",
-    "django.contrib.sitemaps",
-    "django.contrib.sites",
-    "django.contrib.syndication",
-    "django.contrib.webdesign",
-    "south"
-]
-
 def validate_installed_app(app_name, python_path_dir, known_apps, results):
     if app_name in known_apps:
         logger.debug("Application '%s' Ok." % app_name)
@@ -178,7 +164,16 @@ def validate_installed_app(app_name, python_path_dir, known_apps, results):
     for dirname in app_dirs:
         current_dir = os.path.join(current_dir, dirname)
         if not os.path.exists(current_dir):
-            results.warning("Application '%s' not found." % app_name)
+            if app_name in PACKAGES_FOR_KNOWN_APPS:
+                pkgs = PACKAGES_FOR_KNOWN_APPS[app_name]
+                if len(pkgs)>1:
+                    results.warning("Django Application '%s' not found. Perhaps you wanted to include one of the following packages in your requirements.txt file: %s" %
+                                    (app_name, pkgs))
+                else:
+                    results.warning("Django Application '%s' not found. Perhaps you wanted to include the '%s' package in your requirements.txt file" %
+                                    (app_name, pkgs[0]))
+            else:
+                results.warning("Django Application '%s' not found within the application's directory tree. This may be OK if you have an asssociated entry in your requirements.txt file. If it is supposed to be a python package included with you application, check the value you provided for the settings module." % app_name)
             return
     logger.debug("Application '%s' Ok." % app_name)
 
@@ -195,7 +190,8 @@ def validate_post_install_commands(app_name, settings_module, results):
     logger.debug("Post installation commands for application %s Ok." % app_name) 
 
 def validate_fixture_file(fixture_name, installed_apps, fixture_dirs,
-                          python_path_dir, settings_file_directory, results):
+                          python_path_dir, settings_file_directory,
+                          known_apps, results):
     """Use the same logic as django-admin.py loaddata to search for a
     fixture file.
     """
@@ -213,7 +209,7 @@ def validate_fixture_file(fixture_name, installed_apps, fixture_dirs,
         return
     # next, look in the fixtures directory under all the installed apps
     for app_module in installed_apps:
-        if app_module in PREINSTALLED_APPS:
+        if app_module in known_apps:
             continue # we can't check the django apps fixtures
         fixture_dir = os.path.join(app_module_name_to_dir(python_path_dir,
                                                           app_module, False),
@@ -290,7 +286,8 @@ def _get_subdir_component(parent_dir, subdir):
         return subdir[len(parent_dir)+1:]
 
 
-def validate_settings(app_dir_path, django_settings_module, django_config=None):
+def validate_settings(app_dir_path, django_settings_module, django_config=None,
+                      prev_version_component_list=None):
     """This is the main settings validation function. It takes the following arguments:
         app_dir_path           - path to the top level directory of the extracted application
         django_settings_module - fully qualified name of the django settings module
@@ -330,13 +327,12 @@ def validate_settings(app_dir_path, django_settings_module, django_config=None):
     deployed_settings_module = get_deployed_settings_module(django_settings_module)
     logger.debug("Importing settings module %s" % deployed_settings_module)
     try:
-        parent_settings_module = __import__(deployed_settings_module)
-        settings_module = getattr(parent_settings_module, "deployed_settings")
+        settings_module = import_module(deployed_settings_module)
     except:
         (exc_type, exc_value, exc_traceback) = sys.exc_info()
         logger.error("Exception in settings file import: %s" %
                      traceback.format_exception(exc_type, exc_value, exc_traceback))
-        raise SettingsImportError("Error in settings import: %s, %s" % (exc_type, exc_value))
+        raise SettingsImportError("Error in settings import: %s(%s)" % (exc_type, exc_value))
         
     # Check that the settings controlled by engage weren't overridden by app.
     # If any are overridden, we treat them as warnings.
@@ -374,7 +370,8 @@ def validate_settings(app_dir_path, django_settings_module, django_config=None):
     # check that all INSTALLED_APPS are pointing to apps accessible in the target system
     if hasattr(settings_module, "INSTALLED_APPS"):
         installed_apps = []
-        known_apps = set(PREINSTALLED_APPS + get_user_required_packages(app_dir_path))
+        packages = PREINSTALLED_PACKAGES + get_user_required_packages(app_dir_path)
+        known_apps = set(extend_app_list(get_apps_for_packages(packages)))
         for app_name in settings_module.INSTALLED_APPS:
             validate_installed_app(app_name, python_path_dir, known_apps, results)
             installed_apps.append(app_name)
@@ -393,16 +390,19 @@ def validate_settings(app_dir_path, django_settings_module, django_config=None):
         results.fixtures = _tuple_setting_to_list(settings_module.ENGAGE_APP_DB_FIXTURES)
         for fixture in results.fixtures:
             validate_fixture_file(fixture, results.installed_apps, fixture_dirs,
-                                  python_path_dir, settings_file_directory, results)
+                                  python_path_dir, settings_file_directory, known_apps, results)
     else:
         results.fixtures = []
 
     # check ENGAGE_MIGRATION_APPS, if present
     if hasattr(settings_module, "ENGAGE_MIGRATION_APPS"):
         results.migration_apps = _tuple_setting_to_list(settings_module.ENGAGE_MIGRATION_APPS)
+        if len(results.migration_apps)>0 and not ("south" in results.installed_apps):
+            results.error("Django apps to upgraded specified in ENGAGE_MIGRATION_APPS, but south not included in INSTALLED_APPS")
         validate_migration_apps(results.migration_apps, results.installed_apps, results)
     else:
         results.migration_apps = []
+        
     # check each command in ENGAGE_DJANGO_POSTINSTALL_COMMANDS is actually present in manager
     if hasattr(settings_module, "ENGAGE_DJANGO_POSTINSTALL_COMMANDS"):
         results.post_install_commands = list(settings_module.ENGAGE_DJANGO_POSTINSTALL_COMMANDS)
@@ -410,6 +410,13 @@ def validate_settings(app_dir_path, django_settings_module, django_config=None):
     else:
         results.post_install_commands = []
 
+    # check ENGAGE_DJANGO_COMPONENTS
+    if hasattr(settings_module, "ENGAGE_DJANGO_COMPONENTS"):
+        validate_django_components(settings_module.ENGAGE_DJANGO_COMPONENTS,
+                                   prev_version_component_list, results)
+    else:
+        results.components = []
+        
     # extract the product name and version, if present
     if hasattr(settings_module, "ENGAGE_PRODUCT_NAME"):
         results.product = settings_module.ENGAGE_PRODUCT_NAME
@@ -491,9 +498,8 @@ def run_installed_tests_as_subprocess(app_dir_path, django_settings_module, pyth
         use_logger.debug("[%d] %s exited with return code %d" % (subproc.pid,
                                                                  program_and_args[0],
                                                                  subproc.returncode))
-        results_file = open(tf.name, "rb")
-        json_obj = json.load(results_file)
-        results_file.close()
+        with  open(tf.name, "rb") as f:
+            json_obj = json.load(f)
         results = ParsedJsonResults(json_obj)
         return results
     finally:
@@ -566,7 +572,9 @@ class PrepareCommandBase(Command):
                               results.python_path_subdirectory, VERSION,
                               installed_apps=results.installed_apps,
                               fixtures=results.fixtures,
-                              migration_apps=results.migration_apps)
+                              migration_apps=results.migration_apps,
+                              components=results.components,
+                              post_install_commands=results.post_install_commands)
         write_json(config.to_json(), os.path.join(app_dir_path, DJANGO_CFG_FILENAME))
 
         # undo the changes we made
@@ -689,8 +697,8 @@ class ValidateSafeCommand(Command):
             parser.error("Django archive file %s does not exist" % self.django_archive_file)
             
     def run(self):
-        with create_handler(self.django_archive_file) as handler:
-            (common_dir, django_config) = run_safe_validations(handler)
+        (common_dir, django_config) = \
+            run_safe_validations_on_archive(self.django_archive_file)
         logger.info("All safe mode checks passed.\n")
         return SUCCESS_RC
 
