@@ -23,9 +23,8 @@ def app_module_name_to_dir(app_directory_path, app_module_name, check_for_init_p
 
 
 def write_json(json_obj, filename):
-    f = open(filename, 'wb')
-    json.dump(json_obj, f)
-    f.close()
+    with open(filename, 'wb') as f:
+        json.dump(json_obj, f)
 
 
 def find_files(directory, filename_re_pattern, operation_function):
@@ -46,3 +45,18 @@ def get_deployed_settings_module(django_settings_module):
         return "deployed_settings"
     else:
         return '.'.join(mod_comps[0:-1]) + ".deployed_settings"
+
+
+def import_module(qualified_module_name):
+    """Import the specified module and return the contents of that module.
+    For example if we have a module foo.bar containing variables x and y,
+    we can do the following:
+      m = import_module("foo.bar")
+      print m.x, m.y
+    """
+    m = __import__(qualified_module_name)
+    mod_comps = (qualified_module_name.split('.'))[1:]
+    for comp in mod_comps:
+        m = getattr(m, comp)
+    return m
+    
